// Mod25D.cpp : Defines the entry point for the console application.
//

#include "StdAfx.h"

#ifndef _WIN_
#include <unistd.h>
#endif

#ifdef CLUSTER_LAM_MPI

#ifdef _LAM_DEFINE
#undef SEEK_CUR 
#undef SEEK_END
#undef SEEK_SET
#endif
#include <mpi.h>

#include "MpiFunctions.h"

int MPI_Comm_size_our()
{
	int nSize;
	MPI_Comm_size(MPI_COMM_WORLD, &nSize);
	return nSize;
}

int MPI_Comm_rank_our()
{
	int nNode;
	MPI_Comm_rank(MPI_COMM_WORLD, &nNode);
	return nNode;
}

void MPI_Finalize_our()
{
	MPI_Finalize();
}

void MPI_Send_char(char* str, int len, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Send (str,len,MPI_CHAR,nNode,nType,MPI_COMM_WORLD);
}

void MPI_Send_int(int* n, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Send (n,cnt,MPI_INT,nNode,nType,MPI_COMM_WORLD);
}


void MPI_Send_float(float* f, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Send (f,cnt,MPI_FLOAT,nNode,nType,MPI_COMM_WORLD);
}

void MPI_Send_REALNUM(float* f, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Send (f,cnt,MPI_FLOAT,nNode,nType,MPI_COMM_WORLD);
}

void MPI_Send_long(long* l, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Send (l,cnt,MPI_LONG,nNode,nType,MPI_COMM_WORLD);
}

int MPI_Recv_char(char* str, int len, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Status status;
	MPI_Recv (str,len,MPI_CHAR,nNode,nType,MPI_COMM_WORLD,&status);
	return status.MPI_SOURCE;
}

int MPI_Recv_int(int* n, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Status status;
	MPI_Recv (n,cnt,MPI_INT,nNode,nType,MPI_COMM_WORLD,&status);
	return status.MPI_SOURCE;
}

int MPI_Recv_float(float* f, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Status status;
	MPI_Recv (f,cnt,MPI_FLOAT,nNode,nType,MPI_COMM_WORLD,&status);
	return status.MPI_SOURCE;
}

int MPI_Recv_REALNUM(float* f, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Status status;
	MPI_Recv (f,cnt,MPI_FLOAT,nNode,nType,MPI_COMM_WORLD,&status);
	return status.MPI_SOURCE;
}

int MPI_Recv_long(long* l, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Status status;
	MPI_Recv (l,cnt,MPI_LONG,nNode,nType,MPI_COMM_WORLD,&status);
	return status.MPI_SOURCE;
}

void MPI_Barrier_our()
{
	MPI_Barrier(MPI_COMM_WORLD);
}

void MPI_Bcast_int(int* n, int cnt, int nNode)
{
	MPI_Bcast(n,cnt,MPI_INT,nNode,MPI_COMM_WORLD);
}

int MPI_Recv_double(double* f, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Status status;
	MPI_Recv (f,cnt,MPI_DOUBLE,nNode,nType,MPI_COMM_WORLD,&status);
	return status.MPI_SOURCE;
}

void MPI_Send_double(double* f, int cnt, int nNode, int nType)
{
	if(nNode==MPI_ANY_SOURCE_OUR)
		nNode = MPI_ANY_SOURCE;
	MPI_Send (f,cnt,MPI_DOUBLE,nNode,nType,MPI_COMM_WORLD);
}

#endif //CLUSTER_LAM_MPI

int mod25d_run(int nLastShot, int nFirstShot);
extern char g_szExeDir[_MAX_PATH];

int main(int argc, char* argv[])
{
	printf("main started.\n"); fflush(stdout);
//store the Executable directory
	g_szExeDir[0] = '\0';
	
#ifdef _WIN_	
	char* pS = strrchr(argv[0],U_SLASH);
	if(pS!=NULL)
	{
		char* p = argv[0];
		char* q = g_szExeDir;
		while(p<=pS && q-g_szExeDir<_MAX_PATH-1)
			*q++ = *p++;
		*q = '\0';
	}
#else
	int pid = getpid();
	char proc_exe[_MAX_PATH];
	sprintf(proc_exe, "/proc/%d/exe", pid);
	int nr = readlink(proc_exe, g_szExeDir, _MAX_PATH);
	g_szExeDir[nr]=0;
	char* pS = strrchr(g_szExeDir,U_SLASH);
	if(pS!=NULL)
	{
	    pS[0+1]='\0';
	}
#endif
	//printf("%s\n", g_szExeDir);

	int nFirstShot=0, nLastShot=0;
	if(argc==4)
	{
		nFirstShot=atoi(argv[2]); nLastShot=atoi(argv[3]);
	}
///////////
	// Initializing and starting
#ifdef CLUSTER_LAM_MPI
	MPI_Init(&argc, &argv);
	int nNode=MPI_Comm_rank_our();
#endif
	int ret = mod25d_run(nFirstShot,nLastShot);
#ifndef CLUSTER_WIN_OUR
#ifdef CLUSTER_LAM_MPI
	if (nNode==0)
#endif //CLUSTER_LAM_MPI
		printf("The program has finished with result %d.\n",ret);
#endif //CLUSTER_WIN_OUR
#if defined(_WIN_) && !defined(_DEBUG)
	SetUnhandledExceptionFilter( hOldFilter );
#endif
	return ret;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

